@@ -102,6 +102,10 @@ typedef enum {
     VIR_NODE_DEV_CAP_FLAG_PCI_MDEV                  = (1 << 3),
 } virNodeDevPCICapFlags;
 
+typedef enum {
+    VIR_NODE_DEV_CAP_FLAG_CSS_MDEV                  = (1 << 0),
+} virNodeDevCCWCapFlags;
+
 typedef enum {
     /* Keep in sync with VIR_ENUM_IMPL in node_device_conf.c */
     VIR_NODE_DEV_DRM_PRIMARY,
@@ -274,6 +278,9 @@ struct _virNodeDevCapCCW {
     unsigned int cssid;
     unsigned int ssid;
     unsigned int devno;
+    unsigned int flags; /* enum virNodeDevCCWCapFlags */
+    virMediatedDeviceTypePtr *mdev_types;
+    size_t nmdev_types;
 };
 
 typedef struct _virNodeDevCapVDPA virNodeDevCapVDPA;
@@ -391,6 +398,10 @@ int
 virNodeDeviceGetPCIDynamicCaps(const char *sysfsPath,
                                virNodeDevCapPCIDevPtr pci_dev);
 
+int
+virNodeDeviceGetCSSDynamicCaps(const char *sysfsPath,
+                               virNodeDevCapCCWPtr ccw_dev);
+
 int
 virNodeDeviceUpdateCaps(virNodeDeviceDefPtr def);
 
