@@ -59,7 +59,7 @@ VIR_LOG_INIT("locking.lock_daemon");
 struct _virLockDaemon {
     GMutex lock;
     virNetDaemonPtr dmn;
-    virHashTablePtr lockspaces;
+    GHashTable *lockspaces;
     virLockSpacePtr defaultLockspace;
 };
 
@@ -730,7 +730,7 @@ virLockDaemonPreExecRestart(const char *state_file,
     }
 
 
-    tmp = pairs = virHashGetItems(lockDaemon->lockspaces, NULL);
+    tmp = pairs = virHashGetItems(lockDaemon->lockspaces, NULL, false);
     while (tmp && tmp->key) {
         virLockSpacePtr lockspace = (virLockSpacePtr)tmp->value;
 
