@@ -988,7 +988,6 @@ static void
 qemuRestoreCgroupState(virDomainObjPtr vm)
 {
     g_autofree char *mem_mask = NULL;
-    int empty = -1;
     qemuDomainObjPrivatePtr priv = vm->privateData;
     size_t i = 0;
     g_autoptr(virBitmap) all_nodes = NULL;
@@ -1003,8 +1002,8 @@ qemuRestoreCgroupState(virDomainObjPtr vm)
     if (!(mem_mask = virBitmapFormat(all_nodes)))
         goto error;
 
-    if ((empty = virCgroupHasEmptyTasks(priv->cgroup,
-                                        VIR_CGROUP_CONTROLLER_CPUSET)) <= 0)
+    if (virCgroupHasEmptyTasks(priv->cgroup,
+                               VIR_CGROUP_CONTROLLER_CPUSET) <= 0)
         goto error;
 
     if (virCgroupSetCpusetMems(priv->cgroup, mem_mask) < 0)
