@@ -244,7 +244,7 @@ struct _qemuDomainObjPrivate {
     bool memPrealloc;
 
     /* running block jobs */
-    virHashTablePtr blockjobs;
+    GHashTable *blockjobs;
 
     bool disableSlirp;
 
@@ -613,7 +613,7 @@ int qemuDomainSnapshotWriteMetadata(virDomainObjPtr vm,
                                     const char *snapshotDir);
 
 int qemuDomainSnapshotForEachQcow2(virQEMUDriverPtr driver,
-                                   virDomainObjPtr vm,
+                                   virDomainDefPtr def,
                                    virDomainMomentObjPtr snap,
                                    const char *op,
                                    bool try_all);
@@ -780,6 +780,7 @@ bool qemuDomainIsPSeries(const virDomainDef *def);
 bool qemuDomainHasPCIRoot(const virDomainDef *def);
 bool qemuDomainHasPCIeRoot(const virDomainDef *def);
 bool qemuDomainHasBuiltinIDE(const virDomainDef *def);
+bool qemuDomainHasBuiltinESP(const virDomainDef *def);
 bool qemuDomainNeedsFDC(const virDomainDef *def);
 bool qemuDomainSupportsPCI(virDomainDefPtr def,
                            virQEMUCapsPtr qemuCaps);
@@ -1050,3 +1051,8 @@ qemuDomainFileWrapperFDClose(virDomainObjPtr vm,
 int
 qemuDomainInterfaceSetDefaultQDisc(virQEMUDriverPtr driver,
                                    virDomainNetDefPtr net);
+
+int
+qemuDomainNamePathsCleanup(virQEMUDriverConfigPtr cfg,
+                           const char *name,
+                           bool bestEffort);
