@@ -122,12 +122,6 @@ virCgroupV2ValidateMachineGroup(virCgroupPtr group,
     if (!(tmp = strrchr(group->unified.placement, '/')))
         return false;
 
-    if (STREQ(tmp, "/emulator")) {
-        *tmp = '\0';
-
-        if (!(tmp = strrchr(group->unified.placement, '/')))
-            return false;
-    }
     tmp++;
 
     if (STRNEQ(tmp, partmachinename) &&
@@ -155,22 +149,19 @@ virCgroupV2CopyPlacement(virCgroupPtr group,
                          const char *path,
                          virCgroupPtr parent)
 {
+    bool delim = STREQ(parent->unified.placement, "/") || STREQ(path, "");
+
     VIR_DEBUG("group=%p path=%s parent=%p", group, path, parent);
 
-    if (path[0] == '/') {
-        group->unified.placement = g_strdup(path);
-    } else {
-        bool delim = STREQ(parent->unified.placement, "/") || STREQ(path, "");
-        /*
-         * parent == "/" + path="" => "/"
-         * parent == "/libvirt.service" + path == "" => "/libvirt.service"
-         * parent == "/libvirt.service" + path == "foo" => "/libvirt.service/foo"
-         */
-        group->unified.placement = g_strdup_printf("%s%s%s",
-                                                   parent->unified.placement,
-                                                   delim ? "" : "/",
-                                                   path);
-    }
+    /*
+     * parent == "/" + path="" => "/"
+     * parent == "/libvirt.service" + path == "" => "/libvirt.service"
+     * parent == "/libvirt.service" + path == "foo" => "/libvirt.service/foo"
+     */
+    group->unified.placement = g_strdup_printf("%s%s%s",
+                                               parent->unified.placement,
+                                               delim ? "" : "/",
+                                               path);
 
     return 0;
 }
@@ -198,6 +189,9 @@ virCgroupV2DetectPlacement(virCgroupPtr group,
                            const char *controllers,
                            const char *selfpath)
 {
+    g_autofree char *placement = g_strdup(selfpath);
+    char *tmp = NULL;
+
     if (group->unified.placement)
         return 0;
 
@@ -208,18 +202,34 @@ virCgroupV2DetectPlacement(virCgroupPtr group,
     if (STRNEQ(controllers, ""))
         return 0;
 
+    /* Running VM will have the main thread placed in emulator cgroup
+     * but we need to get the main cgroup. */
+    tmp = g_strrstr(placement, "/emulator");
+    if (tmp)
+        *tmp = '\0';
+
     /*
      * selfpath == "/" + path="" -> "/"
      * selfpath == "/libvirt.service" + path == "" -> "/libvirt.service"
      * selfpath == "/libvirt.service" + path == "foo" -> "/libvirt.service/foo"
      */
-    group->unified.placement = g_strdup_printf("%s%s%s", selfpath,
+    group->unified.placement = g_strdup_printf("%s%s%s", placement,
                                                (STREQ(selfpath, "/") || STREQ(path, "") ? "" : "/"), path);
 
     return 0;
 }
 
 
+static int
+virCgroupV2SetPlacement(virCgroupPtr group,
+                        const char *path)
+{
+    group->unified.placement = g_strdup(path);
+
+    return 0;
+}
+
+
 static int
 virCgroupV2ValidatePlacement(virCgroupPtr group,
                              pid_t pid G_GNUC_UNUSED)
@@ -252,7 +262,7 @@ virCgroupV2ParseControllersFile(virCgroupPtr group,
     char **tmp;
 
     if (parent) {
-        contFile = g_strdup_printf("%s%s/cgroup.subtree_control",
+        contFile = g_strdup_printf("%s%s/cgroup.controllers",
                                    parent->unified.mountPoint,
                                    NULLSTR_EMPTY(parent->unified.placement));
     } else {
@@ -410,8 +420,6 @@ virCgroupV2MakeGroup(virCgroupPtr parent,
         return 0;
     }
 
-    VIR_DEBUG("Make group %s", group->path);
-
     controller = virCgroupV2GetAnyController(group);
     if (virCgroupV2PathOfController(group, controller, "", &path) < 0)
         return -1;
@@ -421,7 +429,7 @@ virCgroupV2MakeGroup(virCgroupPtr parent,
     if (!virFileExists(path) &&
         (!create || (mkdir(path, 0755) < 0 && errno != EEXIST))) {
         virReportSystemError(errno, _("Failed to create v2 cgroup '%s'"),
-                             group->path);
+                             path);
         return -1;
     }
 
@@ -521,7 +529,7 @@ virCgroupV2HasEmptyTasks(virCgroupPtr cgroup,
     int ret = -1;
     g_autofree char *content = NULL;
 
-    ret = virCgroupGetValueStr(cgroup, controller, "cgroup.procs", &content);
+    ret = virCgroupGetValueStr(cgroup, controller, "cgroup.threads", &content);
 
     if (ret == 0 && content[0] == '\0')
         ret = 1;
@@ -1844,6 +1852,7 @@ virCgroupBackend virCgroupV2Backend = {
     .copyPlacement = virCgroupV2CopyPlacement,
     .detectMounts = virCgroupV2DetectMounts,
     .detectPlacement = virCgroupV2DetectPlacement,
+    .setPlacement = virCgroupV2SetPlacement,
     .validatePlacement = virCgroupV2ValidatePlacement,
     .stealPlacement = virCgroupV2StealPlacement,
     .detectControllers = virCgroupV2DetectControllers,
