@@ -452,11 +452,10 @@ static int
 virCommandMassCloseGetFDsLinux(virCommandPtr cmd G_GNUC_UNUSED,
                                virBitmapPtr fds)
 {
-    DIR *dp = NULL;
+    g_autoptr(DIR) dp = NULL;
     struct dirent *entry;
     const char *dirName = "/proc/self/fd";
     int rc;
-    int ret = -1;
 
     if (virDirOpen(&dp, dirName) < 0)
         return -1;
@@ -468,19 +467,16 @@ virCommandMassCloseGetFDsLinux(virCommandPtr cmd G_GNUC_UNUSED,
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            _("unable to parse FD: %s"),
                            entry->d_name);
-            goto cleanup;
+            return -1;
         }
 
         ignore_value(virBitmapSetBit(fds, fd));
     }
 
     if (rc < 0)
-        goto cleanup;
+        return -1;
 
-    ret = 0;
- cleanup:
-    VIR_DIR_CLOSE(dp);
-    return ret;
+    return 0;
 }
 
 # else /* !__linux__ */
@@ -1507,7 +1503,7 @@ virCommandAddArg(virCommandPtr cmd, const char *val)
 
     if (val == NULL) {
         cmd->has_error = EINVAL;
-        abort();
+        return;
     }
 
     arg = g_strdup(val);
