@@ -169,9 +169,8 @@ virDMSanitizepath(const char *path)
     g_autofree char *dmDirPath = NULL;
     struct dirent *ent = NULL;
     struct stat sb[2];
-    DIR *dh = NULL;
+    g_autoptr(DIR) dh = NULL;
     const char *p;
-    char *ret = NULL;
 
     /* If a path is NOT provided then assume it's DM name */
     p = strrchr(path, '/');
@@ -205,14 +204,12 @@ virDMSanitizepath(const char *path)
         g_autofree char *tmp = g_strdup_printf(DEV_DM_DIR "/%s", ent->d_name);
 
         if (stat(tmp, &sb[1]) == 0 &&
-            sb[0].st_rdev == sb[0].st_rdev) {
-            ret = g_steal_pointer(&tmp);
-            break;
+            sb[0].st_rdev == sb[1].st_rdev) {
+            return g_steal_pointer(&tmp);
         }
     }
 
-    virDirClose(&dh);
-    return ret;
+    return NULL;
 }
 
 
