@@ -432,7 +432,7 @@ virPCIDeviceIterDevices(virPCIDeviceIterPredicate predicate,
                         virPCIDevicePtr *matched,
                         void *data)
 {
-    DIR *dir;
+    g_autoptr(DIR) dir = NULL;
     struct dirent *entry;
     int ret = 0;
     int rc;
@@ -480,7 +480,6 @@ virPCIDeviceIterDevices(virPCIDeviceIterPredicate predicate,
             break;
         }
     }
-    VIR_DIR_CLOSE(dir);
     return ret;
 }
 
@@ -1706,8 +1705,7 @@ int virPCIDeviceFileIterate(virPCIDevicePtr dev,
                             void *opaque)
 {
     g_autofree char *pcidir = NULL;
-    DIR *dir = NULL;
-    int ret = -1;
+    g_autoptr(DIR) dir = NULL;
     struct dirent *ent;
     int direrr;
 
@@ -1716,7 +1714,7 @@ int virPCIDeviceFileIterate(virPCIDevicePtr dev,
                              dev->address.function);
 
     if (virDirOpen(&dir, pcidir) < 0)
-        goto cleanup;
+        return -1;
 
     while ((direrr = virDirRead(dir, &ent, pcidir)) > 0) {
         g_autofree char *file = NULL;
@@ -1732,17 +1730,13 @@ int virPCIDeviceFileIterate(virPCIDevicePtr dev,
             STREQ(ent->d_name, "reset")) {
             file = g_strdup_printf("%s/%s", pcidir, ent->d_name);
             if ((actor)(dev, file, opaque) < 0)
-                goto cleanup;
+                return -1;
         }
     }
     if (direrr < 0)
-        goto cleanup;
+        return -1;
 
-    ret = 0;
-
- cleanup:
-    VIR_DIR_CLOSE(dir);
-    return ret;
+    return 0;
 }
 
 
@@ -1757,8 +1751,7 @@ virPCIDeviceAddressIOMMUGroupIterate(virPCIDeviceAddressPtr orig,
                                      void *opaque)
 {
     g_autofree char *groupPath = NULL;
-    DIR *groupDir = NULL;
-    int ret = -1;
+    g_autoptr(DIR) groupDir = NULL;
     struct dirent *ent;
     int direrr;
 
@@ -1767,8 +1760,7 @@ virPCIDeviceAddressIOMMUGroupIterate(virPCIDeviceAddressPtr orig,
 
     if (virDirOpenQuiet(&groupDir, groupPath) < 0) {
         /* just process the original device, nothing more */
-        ret = (actor)(orig, opaque);
-        goto cleanup;
+        return (actor)(orig, opaque);
     }
 
     while ((direrr = virDirRead(groupDir, &ent, groupPath)) > 0) {
@@ -1778,20 +1770,16 @@ virPCIDeviceAddressIOMMUGroupIterate(virPCIDeviceAddressPtr orig,
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            _("Found invalid device link '%s' in '%s'"),
                            ent->d_name, groupPath);
-            goto cleanup;
+            return -1;
         }
 
         if ((actor)(&newDev, opaque) < 0)
-            goto cleanup;
+            return -1;
     }
     if (direrr < 0)
-        goto cleanup;
+        return -1;
 
-    ret = 0;
-
- cleanup:
-    VIR_DIR_CLOSE(groupDir);
-    return ret;
+    return 0;
 }
 
 
@@ -2387,8 +2375,7 @@ virPCIGetNetName(const char *device_link_sysfs_path,
 {
     g_autofree char *pcidev_sysfs_net_path = NULL;
     g_autofree char *firstEntryName = NULL;
-    int ret = -1;
-    DIR *dir = NULL;
+    g_autoptr(DIR) dir = NULL;
     struct dirent *entry = NULL;
     size_t i = 0;
 
@@ -2402,8 +2389,7 @@ virPCIGetNetName(const char *device_link_sysfs_path,
 
     if (virDirOpenQuiet(&dir, pcidev_sysfs_net_path) < 0) {
         /* this *isn't* an error - caller needs to check for netname == NULL */
-        ret = 0;
-        goto cleanup;
+        return 0;
     }
 
     while (virDirRead(dir, &entry, pcidev_sysfs_net_path) > 0) {
@@ -2414,7 +2400,7 @@ virPCIGetNetName(const char *device_link_sysfs_path,
             g_autofree char *thisPhysPortID = NULL;
 
             if (virNetDevGetPhysPortID(entry->d_name, &thisPhysPortID) < 0)
-                goto cleanup;
+                return -1;
 
             /* if this one doesn't match, keep looking */
             if (STRNEQ_NULLABLE(physPortID, thisPhysPortID)) {
@@ -2434,35 +2420,27 @@ virPCIGetNetName(const char *device_link_sysfs_path,
         }
 
         *netname = g_strdup(entry->d_name);
-
-        ret = 0;
-        break;
+        return 0;
     }
 
-    if (ret < 0) {
-        if (physPortID) {
-            if (firstEntryName) {
-                /* we didn't match the provided phys_port_id, but this
-                 * is probably because phys_port_id isn't implemented
-                 * for this NIC driver, so just return the first
-                 * (probably only) netname we found.
-                 */
-                *netname = firstEntryName;
-                firstEntryName = NULL;
-                ret = 0;
-            } else {
-                virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Could not find network device with "
-                                 "phys_port_id '%s' under PCI device at %s"),
-                               physPortID, device_link_sysfs_path);
-            }
-        } else {
-            ret = 0; /* no netdev at the given index is *not* an error */
-        }
+    if (!physPortID)
+        return 0;
+
+    if (firstEntryName) {
+        /* we didn't match the provided phys_port_id, but this
+         * is probably because phys_port_id isn't implemented
+         * for this NIC driver, so just return the first
+         * (probably only) netname we found.
+         */
+        *netname = g_steal_pointer(&firstEntryName);
+        return 0;
     }
- cleanup:
-    VIR_DIR_CLOSE(dir);
-    return ret;
+
+    virReportError(VIR_ERR_INTERNAL_ERROR,
+                   _("Could not find network device with "
+                     "phys_port_id '%s' under PCI device at %s"),
+                   physPortID, device_link_sysfs_path);
+    return -1;
 }
 
 int
@@ -2528,57 +2506,6 @@ virPCIGetVirtualFunctionInfo(const char *vf_sysfs_device_path,
     return 0;
 }
 
-
-ssize_t
-virPCIGetMdevTypes(const char *sysfspath,
-                   virMediatedDeviceTypePtr **types)
-{
-    ssize_t ret = -1;
-    int dirret = -1;
-    DIR *dir = NULL;
-    struct dirent *entry;
-    g_autofree char *types_path = NULL;
-    g_autoptr(virMediatedDeviceType) mdev_type = NULL;
-    virMediatedDeviceTypePtr *mdev_types = NULL;
-    size_t ntypes = 0;
-    size_t i;
-
-    types_path = g_strdup_printf("%s/mdev_supported_types", sysfspath);
-
-    if ((dirret = virDirOpenIfExists(&dir, types_path)) < 0)
-        goto cleanup;
-
-    if (dirret == 0) {
-        ret = 0;
-        goto cleanup;
-    }
-
-    while ((dirret = virDirRead(dir, &entry, types_path)) > 0) {
-        g_autofree char *tmppath = NULL;
-        /* append the type id to the path and read the attributes from there */
-        tmppath = g_strdup_printf("%s/%s", types_path, entry->d_name);
-
-        if (virMediatedDeviceTypeReadAttrs(tmppath, &mdev_type) < 0)
-            goto cleanup;
-
-        if (VIR_APPEND_ELEMENT(mdev_types, ntypes, mdev_type) < 0)
-            goto cleanup;
-    }
-
-    if (dirret < 0)
-        goto cleanup;
-
-    *types = g_steal_pointer(&mdev_types);
-    ret = ntypes;
-    ntypes = 0;
- cleanup:
-    for (i = 0; i < ntypes; i++)
-        virMediatedDeviceTypeFree(mdev_types[i]);
-    VIR_FREE(mdev_types);
-    VIR_DIR_CLOSE(dir);
-    return ret;
-}
-
 #else
 static const char *unsupported = N_("not supported on non-linux platforms");
 
@@ -2653,15 +2580,6 @@ virPCIGetVirtualFunctionInfo(const char *vf_sysfs_device_path G_GNUC_UNUSED,
     virReportError(VIR_ERR_INTERNAL_ERROR, "%s", _(unsupported));
     return -1;
 }
-
-
-ssize_t
-virPCIGetMdevTypes(const char *sysfspath G_GNUC_UNUSED,
-                   virMediatedDeviceTypePtr **types G_GNUC_UNUSED)
-{
-    virReportError(VIR_ERR_INTERNAL_ERROR, "%s", _(unsupported));
-    return -1;
-}
 #endif /* __linux__ */
 
 int
