@@ -781,29 +781,23 @@ virResctrlGetMonitorInfo(virResctrlInfoPtr resctrl)
 static int
 virResctrlGetInfo(virResctrlInfoPtr resctrl)
 {
-    DIR *dirp = NULL;
+    g_autoptr(DIR) dirp = NULL;
     int ret = -1;
 
     ret = virDirOpenIfExists(&dirp, SYSFS_RESCTRL_PATH "/info");
     if (ret <= 0)
-        goto cleanup;
+        return ret;
 
-    ret = virResctrlGetMemoryBandwidthInfo(resctrl);
-    if (ret < 0)
-        goto cleanup;
+    if ((ret = virResctrlGetMemoryBandwidthInfo(resctrl)) < 0)
+        return -1;
 
-    ret = virResctrlGetCacheInfo(resctrl, dirp);
-    if (ret < 0)
-        goto cleanup;
+    if ((ret = virResctrlGetCacheInfo(resctrl, dirp)) < 0)
+        return -1;
 
-    ret = virResctrlGetMonitorInfo(resctrl);
-    if (ret < 0)
-        goto cleanup;
+    if ((ret = virResctrlGetMonitorInfo(resctrl)) < 0)
+        return -1;
 
-    ret = 0;
- cleanup:
-    VIR_DIR_CLOSE(dirp);
-    return ret;
+    return 0;
 }
 
 
@@ -1900,7 +1894,7 @@ virResctrlAllocGetUnused(virResctrlInfoPtr resctrl)
     virResctrlAllocPtr ret = NULL;
     virResctrlAllocPtr alloc = NULL;
     struct dirent *ent = NULL;
-    DIR *dirp = NULL;
+    g_autoptr(DIR) dirp = NULL;
     int rv = -1;
 
     if (virResctrlInfoIsEmpty(resctrl)) {
@@ -1947,7 +1941,6 @@ virResctrlAllocGetUnused(virResctrlInfoPtr resctrl)
 
  cleanup:
     virObjectUnref(alloc);
-    VIR_DIR_CLOSE(dirp);
     return ret;
 
  error:
@@ -2663,7 +2656,7 @@ virResctrlMonitorGetStats(virResctrlMonitorPtr monitor,
     int ret = -1;
     size_t i = 0;
     unsigned long long val = 0;
-    DIR *dirp = NULL;
+    g_autoptr(DIR) dirp = NULL;
     char *datapath = NULL;
     char *filepath = NULL;
     struct dirent *ent = NULL;
@@ -2747,7 +2740,6 @@ virResctrlMonitorGetStats(virResctrlMonitorPtr monitor,
     VIR_FREE(datapath);
     VIR_FREE(filepath);
     virResctrlMonitorStatsFree(stat);
-    VIR_DIR_CLOSE(dirp);
     return ret;
 }
 
