@@ -22,6 +22,7 @@
 #include "testutils.h"
 #include "testutilsqemu.h"
 #include "security/security_manager.h"
+#include "security/security_util.h"
 #include "conf/domain_conf.h"
 #include "qemu/qemu_domain.h"
 #include "qemu/qemu_security.h"
@@ -73,6 +74,9 @@ prepareObjects(virQEMUDriverPtr driver,
                                             0)))
         return -1;
 
+    if (virSecurityManagerGenLabel(driver->securityManager, vm->def) < 0)
+        return -1;
+
     *vm_ret = g_steal_pointer(&vm);
     return 0;
 }
@@ -134,23 +138,63 @@ static int
 mymain(void)
 {
     virQEMUDriver driver;
+    virSecurityManagerPtr stack = NULL;
+    virSecurityManagerPtr dac = NULL;
+#ifdef WITH_SELINUX
+    virSecurityManagerPtr selinux = NULL;
+#endif
     int ret = 0;
 
     if (virInitialize() < 0 ||
         qemuTestDriverInit(&driver) < 0)
         return -1;
 
+    if (!virSecurityXATTRNamespaceDefined()) {
+        ret = EXIT_AM_SKIP;
+        goto cleanup;
+    }
+
     /* Now fix the secdriver */
     virObjectUnref(driver.securityManager);
-    if (!(driver.securityManager = virSecurityManagerNewDAC("test", 1000, 1000,
-                                                            VIR_SECURITY_MANAGER_PRIVILEGED |
-                                                            VIR_SECURITY_MANAGER_DYNAMIC_OWNERSHIP,
-                                                            NULL))) {
+
+    if (!(dac = virSecurityManagerNewDAC("test", 1000, 1000,
+                                         VIR_SECURITY_MANAGER_PRIVILEGED |
+                                         VIR_SECURITY_MANAGER_DYNAMIC_OWNERSHIP,
+                                         NULL))) {
         fprintf(stderr, "Cannot initialize DAC security driver");
         ret = -1;
         goto cleanup;
     }
 
+    if (!(stack = virSecurityManagerNewStack(dac))) {
+        fprintf(stderr, "Cannot initialize stack security driver");
+        ret = -1;
+        goto cleanup;
+    }
+    dac = NULL;
+
+#if WITH_SELINUX
+    selinux = virSecurityManagerNew("selinux", "test",
+                                    VIR_SECURITY_MANAGER_PRIVILEGED |
+                                    VIR_SECURITY_MANAGER_DEFAULT_CONFINED |
+                                    VIR_SECURITY_MANAGER_REQUIRE_CONFINED);
+    if (!selinux) {
+        fprintf(stderr, "Cannot initialize selinux security driver");
+        ret = -1;
+        goto cleanup;
+    }
+
+    if (virSecurityManagerStackAddNested(stack, selinux) < 0) {
+        fprintf(stderr, "Cannot add selinux security driver onto stack");
+        ret = -1;
+        goto cleanup;
+    }
+    selinux = NULL;
+#endif
+
+    driver.securityManager = g_steal_pointer(&stack);
+
+
 #define DO_TEST_DOMAIN(f) \
     do { \
         struct testData data = {.driver = &driver, .file = f}; \
@@ -214,6 +258,11 @@ mymain(void)
 
  cleanup:
     qemuTestDriverFree(&driver);
+#ifdef WITH_SELINUX
+    virObjectUnref(selinux);
+#endif
+    virObjectUnref(dac);
+    virObjectUnref(stack);
     return ret;
 }
 
