@@ -7,17 +7,15 @@
 
 #include "testutils.h"
 
-#ifdef WITH_QEMU
+#include "internal.h"
+#include "qemu/qemu_domain_address.h"
+#include "qemu/qemu_domain.h"
+#include "testutilsqemu.h"
+#include "virstring.h"
+#include "virfilewrapper.h"
+#include "configmake.h"
 
-# include "internal.h"
-# include "qemu/qemu_domain_address.h"
-# include "qemu/qemu_domain.h"
-# include "testutilsqemu.h"
-# include "virstring.h"
-# include "virfilewrapper.h"
-# include "configmake.h"
-
-# define VIR_FROM_THIS VIR_FROM_NONE
+#define VIR_FROM_THIS VIR_FROM_NONE
 
 static virQEMUDriver driver;
 
@@ -50,45 +48,6 @@ testXML2XMLInactive(const void *opaque)
 }
 
 
-static int
-testCompareStatusXMLToXMLFiles(const void *opaque)
-{
-    const struct testQemuInfo *data = opaque;
-    virDomainObjPtr obj = NULL;
-    g_autofree char *actual = NULL;
-    int ret = -1;
-
-    if (!(obj = virDomainObjParseFile(data->infile, driver.xmlopt,
-                                      VIR_DOMAIN_DEF_PARSE_STATUS |
-                                      VIR_DOMAIN_DEF_PARSE_ACTUAL_NET |
-                                      VIR_DOMAIN_DEF_PARSE_PCI_ORIG_STATES |
-                                      VIR_DOMAIN_DEF_PARSE_SKIP_VALIDATE |
-                                      VIR_DOMAIN_DEF_PARSE_ALLOW_POST_PARSE_FAIL))) {
-        VIR_TEST_DEBUG("\nfailed to parse '%s'", data->infile);
-        goto cleanup;
-    }
-
-    if (!(actual = virDomainObjFormat(obj, driver.xmlopt,
-                                      VIR_DOMAIN_DEF_FORMAT_SECURE |
-                                      VIR_DOMAIN_DEF_FORMAT_STATUS |
-                                      VIR_DOMAIN_DEF_FORMAT_ACTUAL_NET |
-                                      VIR_DOMAIN_DEF_FORMAT_PCI_ORIG_STATES |
-                                      VIR_DOMAIN_DEF_FORMAT_CLOCK_ADJUST))) {
-        VIR_TEST_DEBUG("\nfailed to format back '%s'", data->infile);
-        goto cleanup;
-    }
-
-    if (virTestCompareToFile(actual, data->outfile) < 0)
-        goto cleanup;
-
-    ret = 0;
-
- cleanup:
-    virDomainObjEndAPI(&obj);
-    return ret;
-}
-
-
 static int
 testInfoSetPaths(struct testQemuInfo *info,
                  const char *suffix,
@@ -115,17 +74,7 @@ testInfoSetPaths(struct testQemuInfo *info,
 }
 
 
-static const char *statusPath = abs_srcdir "/qemustatusxml2xmldata/";
-
-static void
-testInfoSetStatusPaths(struct testQemuInfo *info)
-{
-    info->infile = g_strdup_printf("%s%s-in.xml", statusPath, info->name);
-    info->outfile = g_strdup_printf("%s%s-out.xml", statusPath, info->name);
-}
-
-
-# define FAKEROOTDIRTEMPLATE abs_builddir "/fakerootdir-XXXXXX"
+#define FAKEROOTDIRTEMPLATE abs_builddir "/fakerootdir-XXXXXX"
 
 static int
 mymain(void)
@@ -133,7 +82,7 @@ mymain(void)
     int ret = 0;
     g_autofree char *fakerootdir = NULL;
     g_autoptr(virQEMUDriverConfig) cfg = NULL;
-    g_autoptr(virHashTable) capslatest = NULL;
+    g_autoptr(GHashTable) capslatest = NULL;
     g_autoptr(virConnect) conn = NULL;
 
     capslatest = testQemuGetLatestCaps();
@@ -174,7 +123,7 @@ mymain(void)
     virSetConnectSecret(conn);
     virSetConnectStorage(conn);
 
-# define DO_TEST_INTERNAL(_name, suffix, when, ...) \
+#define DO_TEST_INTERNAL(_name, suffix, when, ...) \
     do { \
         static struct testQemuInfo info = { \
             .name = _name, \
@@ -209,38 +158,38 @@ mymain(void)
         testQemuInfoClear(&info); \
     } while (0)
 
-# define DO_TEST_CAPS_INTERNAL(name, arch, ver, ...) \
+#define DO_TEST_CAPS_INTERNAL(name, arch, ver, ...) \
     DO_TEST_INTERNAL(name, "." arch "-" ver, WHEN_BOTH, \
                      ARG_CAPS_ARCH, arch, \
                      ARG_CAPS_VER, ver, \
                      __VA_ARGS__)
 
-# define DO_TEST_CAPS_ARCH_LATEST_FULL(name, arch, ...) \
+#define DO_TEST_CAPS_ARCH_LATEST_FULL(name, arch, ...) \
     DO_TEST_CAPS_INTERNAL(name, arch, "latest", __VA_ARGS__)
 
-# define DO_TEST_CAPS_ARCH_VER_FULL(name, arch, ver, ...) \
+#define DO_TEST_CAPS_ARCH_VER_FULL(name, arch, ver, ...) \
     DO_TEST_CAPS_INTERNAL(name, arch, ver, __VA_ARGS__)
 
-# define DO_TEST_CAPS_ARCH_LATEST(name, arch) \
+#define DO_TEST_CAPS_ARCH_LATEST(name, arch) \
     DO_TEST_CAPS_ARCH_LATEST_FULL(name, arch, ARG_END)
 
-# define DO_TEST_CAPS_ARCH_VER(name, arch, ver) \
+#define DO_TEST_CAPS_ARCH_VER(name, arch, ver) \
     DO_TEST_CAPS_ARCH_VER_FULL(name, arch, ver, ARG_END)
 
-# define DO_TEST_CAPS_LATEST(name) \
+#define DO_TEST_CAPS_LATEST(name) \
     DO_TEST_CAPS_ARCH_LATEST(name, "x86_64")
 
-# define DO_TEST_CAPS_VER(name, ver) \
+#define DO_TEST_CAPS_VER(name, ver) \
     DO_TEST_CAPS_ARCH_VER(name, "x86_64", ver)
 
-# define DO_TEST_FULL(name, when, ...) \
+#define DO_TEST_FULL(name, when, ...) \
     DO_TEST_INTERNAL(name, "", when, __VA_ARGS__)
 
-# define DO_TEST(name, ...) \
+#define DO_TEST(name, ...) \
     DO_TEST_FULL(name, WHEN_BOTH, \
                  ARG_QEMU_CAPS, __VA_ARGS__, QEMU_CAPS_LAST)
 
-# define NONE QEMU_CAPS_LAST
+#define NONE QEMU_CAPS_LAST
 
     /* Unset or set all envvars here that are copied in qemudBuildCommandLine
      * using ADD_ENV_COPY, otherwise these tests may fail due to unexpected
@@ -1423,41 +1372,6 @@ mymain(void)
             QEMU_CAPS_VIRTIO_SCSI,
             QEMU_CAPS_MCH_EXTENDED_TSEG_MBYTES);
 
-# define DO_TEST_STATUS(_name) \
-    do { \
-        static struct testQemuInfo info = { \
-            .name = _name, \
-        }; \
-        if (testQemuInfoSetArgs(&info, capslatest, \
-                                ARG_QEMU_CAPS, QEMU_CAPS_LAST, \
-                                ARG_END) < 0 || \
-            qemuTestCapsCacheInsert(driver.qemuCapsCache, info.qemuCaps) < 0) { \
-            VIR_TEST_DEBUG("Failed to generate status test data for '%s'", _name); \
-            return -1; \
-        } \
-        testInfoSetStatusPaths(&info); \
-\
-        if (virTestRun("QEMU status XML-2-XML " _name, \
-                       testCompareStatusXMLToXMLFiles, &info) < 0) \
-            ret = -1; \
-\
-        testQemuInfoClear(&info); \
-    } while (0)
-
-
-    DO_TEST_STATUS("blockjob-mirror");
-    DO_TEST_STATUS("vcpus-multi");
-    DO_TEST_STATUS("modern");
-    DO_TEST_STATUS("migration-out-nbd");
-    DO_TEST_STATUS("migration-in-params");
-    DO_TEST_STATUS("migration-out-params");
-    DO_TEST_STATUS("migration-out-nbd-tls");
-    DO_TEST_STATUS("upgrade");
-
-    DO_TEST_STATUS("blockjob-blockdev");
-
-    DO_TEST_STATUS("backup-pull");
-
     DO_TEST("vhost-vsock", QEMU_CAPS_DEVICE_VHOST_VSOCK);
     DO_TEST("vhost-vsock-auto", QEMU_CAPS_DEVICE_VHOST_VSOCK);
     DO_TEST("vhost-vsock-ccw", QEMU_CAPS_DEVICE_VHOST_VSOCK,
@@ -1513,6 +1427,7 @@ mymain(void)
     DO_TEST_CAPS_ARCH_LATEST("x86_64-default-cpu-tcg-q35-4.2", "x86_64");
 
     DO_TEST_CAPS_LATEST("virtio-9p-multidevs");
+    DO_TEST_CAPS_LATEST("virtio-9p-createmode");
     DO_TEST("downscript", NONE);
 
  cleanup:
@@ -1528,15 +1443,4 @@ mymain(void)
 VIR_TEST_MAIN_PRELOAD(mymain,
                       VIR_TEST_MOCK("virpci"),
                       VIR_TEST_MOCK("virrandom"),
-                      VIR_TEST_MOCK("domaincaps"),
-                      VIR_TEST_MOCK("virdeterministichash"))
-
-#else
-
-int
-main(void)
-{
-    return EXIT_AM_SKIP;
-}
-
-#endif /* WITH_QEMU */
+                      VIR_TEST_MOCK("domaincaps"))
