@@ -31,7 +31,8 @@ static const char *qemu_emulators[VIR_ARCH_LAST] = {
     [VIR_ARCH_PPC] = "/usr/bin/qemu-system-ppc",
     [VIR_ARCH_RISCV32] = "/usr/bin/qemu-system-riscv32",
     [VIR_ARCH_RISCV64] = "/usr/bin/qemu-system-riscv64",
-    [VIR_ARCH_S390X] = "/usr/bin/qemu-system-s390x"
+    [VIR_ARCH_S390X] = "/usr/bin/qemu-system-s390x",
+    [VIR_ARCH_SPARC] = "/usr/bin/qemu-system-sparc",
 };
 
 static const virArch arch_alias[VIR_ARCH_LAST] = {
@@ -74,6 +75,11 @@ static const char *const riscv64_machines[] = {
 static const char *const s390x_machines[] = {
     "s390-virtio", "s390-ccw-virtio", "s390-ccw", NULL
 };
+static const char *const sparc_machines[] = {
+    "SS-5", "LX", "SPARCClassic", "SPARCbook",
+    "SS-10", "SS-20", "SS-4", "SS-600MP",
+    "Voyager", "leon3_generic", NULL
+};
 
 static const char *const *qemu_machines[VIR_ARCH_LAST] = {
     [VIR_ARCH_I686] = i386_machines,
@@ -85,6 +91,7 @@ static const char *const *qemu_machines[VIR_ARCH_LAST] = {
     [VIR_ARCH_RISCV32] = riscv32_machines,
     [VIR_ARCH_RISCV64] = riscv64_machines,
     [VIR_ARCH_S390X] = s390x_machines,
+    [VIR_ARCH_SPARC] = sparc_machines,
 };
 
 static const char *const *kvm_machines[VIR_ARCH_LAST] = {
@@ -106,7 +113,8 @@ static const char *qemu_default_ram_id[VIR_ARCH_LAST] = {
     [VIR_ARCH_ARMV7L] = "vexpress.highmem",
     [VIR_ARCH_PPC64] = "ppc_spapr.ram",
     [VIR_ARCH_PPC] = "ppc_spapr.ram",
-    [VIR_ARCH_S390X] = "s390.ram"
+    [VIR_ARCH_S390X] = "s390.ram",
+    [VIR_ARCH_SPARC] = "sun4m.ram",
 };
 
 char *
@@ -181,19 +189,21 @@ testQemuAddGuest(virCapsPtr caps,
                                        NULL))
         goto error;
 
-    nmachines = g_strv_length((char **)kvm_machines[emu_arch]);
-    machines = virCapabilitiesAllocMachines(kvm_machines[emu_arch],
-                                            nmachines);
-    if (machines == NULL)
-        goto error;
+    if (kvm_machines[emu_arch] != NULL) {
+        nmachines = g_strv_length((char **)kvm_machines[emu_arch]);
+        machines = virCapabilitiesAllocMachines(kvm_machines[emu_arch],
+                                                nmachines);
+        if (machines == NULL)
+            goto error;
 
-    if (!virCapabilitiesAddGuestDomain(guest,
-                                       VIR_DOMAIN_VIRT_KVM,
-                                       qemu_emulators[emu_arch],
-                                       NULL,
-                                       nmachines,
-                                       machines))
-        goto error;
+        if (!virCapabilitiesAddGuestDomain(guest,
+                                           VIR_DOMAIN_VIRT_KVM,
+                                           qemu_emulators[emu_arch],
+                                           NULL,
+                                           nmachines,
+                                           machines))
+            goto error;
+    }
 
     return 0;
 
@@ -363,18 +373,20 @@ int qemuTestCapsCacheInsert(virFileCachePtr cache,
                                       defaultRAMid);
                 virQEMUCapsSet(tmpCaps, QEMU_CAPS_TCG);
             }
-            for (j = 0; kvm_machines[i][j] != NULL; j++) {
-                virQEMUCapsAddMachine(tmpCaps,
-                                      VIR_DOMAIN_VIRT_KVM,
-                                      kvm_machines[i][j],
-                                      NULL,
-                                      NULL,
-                                      0,
+            if (kvm_machines[i] != NULL) {
+                for (j = 0; kvm_machines[i][j] != NULL; j++) {
+                    virQEMUCapsAddMachine(tmpCaps,
+                                          VIR_DOMAIN_VIRT_KVM,
+                                          kvm_machines[i][j],
+                                          NULL,
+                                          NULL,
+                                          0,
+                                          false,
                                       false,
-                                      false,
-                                      true,
-                                      defaultRAMid);
-                virQEMUCapsSet(tmpCaps, QEMU_CAPS_KVM);
+                                          true,
+                                          defaultRAMid);
+                    virQEMUCapsSet(tmpCaps, QEMU_CAPS_KVM);
+                }
             }
         }
 
@@ -510,18 +522,17 @@ testQemuGetLatestCapsForArch(const char *arch,
                              const char *suffix)
 {
     struct dirent *ent;
-    DIR *dir = NULL;
+    g_autoptr(DIR) dir = NULL;
     int rc;
     g_autofree char *fullsuffix = NULL;
     unsigned long maxver = 0;
     unsigned long ver;
     g_autofree char *maxname = NULL;
-    char *ret = NULL;
 
     fullsuffix = g_strdup_printf("%s.%s", arch, suffix);
 
     if (virDirOpen(&dir, TEST_QEMU_CAPS_PATH) < 0)
-        goto cleanup;
+        return NULL;
 
     while ((rc = virDirRead(dir, &ent, TEST_QEMU_CAPS_PATH)) > 0) {
         g_autofree char *tmp = NULL;
@@ -547,23 +558,19 @@ testQemuGetLatestCapsForArch(const char *arch,
     }
 
     if (rc < 0)
-        goto cleanup;
+        return NULL;
 
     if (!maxname) {
         VIR_TEST_VERBOSE("failed to find capabilities for '%s' in '%s'",
                          arch, TEST_QEMU_CAPS_PATH);
-        goto cleanup;
+        return NULL;
     }
 
-    ret = g_strdup_printf("%s/%s", TEST_QEMU_CAPS_PATH, maxname);
-
- cleanup:
-    virDirClose(&dir);
-    return ret;
+    return g_strdup_printf("%s/%s", TEST_QEMU_CAPS_PATH, maxname);
 }
 
 
-virHashTablePtr
+GHashTable *
 testQemuGetLatestCaps(void)
 {
     const char *archs[] = {
@@ -573,7 +580,7 @@ testQemuGetLatestCaps(void)
         "s390x",
         "x86_64",
     };
-    virHashTablePtr capslatest;
+    GHashTable *capslatest;
     size_t i;
 
     if (!(capslatest = virHashNew(g_free)))
@@ -606,9 +613,8 @@ testQemuCapsIterate(const char *suffix,
                     void *opaque)
 {
     struct dirent *ent;
-    DIR *dir = NULL;
+    g_autoptr(DIR) dir = NULL;
     int rc;
-    int ret = -1;
     bool fail = false;
 
     if (!callback)
@@ -617,11 +623,11 @@ testQemuCapsIterate(const char *suffix,
     /* Validate suffix */
     if (!STRPREFIX(suffix, ".")) {
         VIR_TEST_VERBOSE("malformed suffix '%s'", suffix);
-        goto cleanup;
+        return -1;
     }
 
     if (virDirOpen(&dir, TEST_QEMU_CAPS_PATH) < 0)
-        goto cleanup;
+        return -1;
 
     while ((rc = virDirRead(dir, &ent, TEST_QEMU_CAPS_PATH)) > 0) {
         g_autofree char *tmp = g_strdup(ent->d_name);
@@ -635,13 +641,13 @@ testQemuCapsIterate(const char *suffix,
         /* Strip the leading prefix */
         if (!(version = STRSKIP(tmp, "caps_"))) {
             VIR_TEST_VERBOSE("malformed file name '%s'", ent->d_name);
-            goto cleanup;
+            return -1;
         }
 
         /* Find the last dot */
         if (!(archName = strrchr(tmp, '.'))) {
             VIR_TEST_VERBOSE("malformed file name '%s'", ent->d_name);
-            goto cleanup;
+            return -1;
         }
 
         /* The version number and the architecture name are separated by
@@ -662,20 +668,15 @@ testQemuCapsIterate(const char *suffix,
     }
 
     if (rc < 0 || fail)
-        goto cleanup;
+        return -1;
 
-    ret = 0;
-
- cleanup:
-    virDirClose(&dir);
-
-    return ret;
+    return 0;
 }
 
 
 int
 testQemuInfoSetArgs(struct testQemuInfo *info,
-                    virHashTablePtr capslatest, ...)
+                    GHashTable *capslatest, ...)
 {
     va_list argptr;
     testQemuInfoArgName argname;
