@@ -30,6 +30,7 @@
 #include "virstring.h"
 #include "virxml.h"
 #include "virperf.h"
+#include "virbitmap.h"
 
 char **
 virshDomainNameCompleter(vshControl *ctl,
@@ -581,3 +582,199 @@ virshDomainCpulistCompleter(vshControl *ctl,
 
     return virshCommaStringListComplete(cpuid, (const char **)cpulist);
 }
+
+
+char **
+virshDomainVcpulistViaAgentCompleter(vshControl *ctl,
+                                     const vshCmd *cmd,
+                                     unsigned int flags)
+{
+    virDomainPtr dom;
+    bool enable = vshCommandOptBool(cmd, "enable");
+    bool disable = vshCommandOptBool(cmd, "disable");
+    virTypedParameterPtr params = NULL;
+    unsigned int nparams = 0;
+    size_t i;
+    int nvcpus;
+    VIR_AUTOSTRINGLIST cpulist = NULL;
+    const char *vcpuid = NULL;
+    char **ret = NULL;
+
+    virCheckFlags(0, NULL);
+
+    if (!(dom = virshCommandOptDomain(ctl, cmd, NULL)))
+        return NULL;
+
+    if (vshCommandOptStringQuiet(ctl, cmd, "cpulist", &vcpuid) < 0)
+        goto cleanup;
+
+    /* retrieve vcpu count from the guest instead of the hypervisor */
+    if ((nvcpus = virDomainGetVcpusFlags(dom,
+                                         VIR_DOMAIN_VCPU_GUEST |
+                                         VIR_DOMAIN_VCPU_MAXIMUM)) < 0)
+        goto cleanup;
+
+    if (!enable && !disable) {
+        cpulist = g_new0(char *, nvcpus + 1);
+        for (i = 0; i < nvcpus; i++)
+            cpulist[i] = g_strdup_printf("%zu", i);
+    } else {
+        g_autofree char *onlineVcpuStr = NULL;
+        g_autofree unsigned char *vcpumap = NULL;
+        g_autoptr(virBitmap) vcpus = NULL;
+        size_t offset = 0;
+        int dummy;
+
+        if (virDomainGetGuestVcpus(dom, &params, &nparams, 0) < 0)
+            goto cleanup;
+
+        onlineVcpuStr = vshGetTypedParamValue(ctl, &params[1]);
+        if (virBitmapParse(onlineVcpuStr, &vcpus, nvcpus) < 0)
+            goto cleanup;
+
+        if (virBitmapToData(vcpus, &vcpumap, &dummy) < 0)
+            goto cleanup;
+
+        if (enable) {
+            cpulist = g_new0(char *, nvcpus - virBitmapCountBits(vcpus) + 1);
+            for (i = 0; i < nvcpus; i++) {
+                if (VIR_CPU_USED(vcpumap, i) != 0)
+                    continue;
+
+                cpulist[offset++] = g_strdup_printf("%zu", i);
+            }
+        } else if (disable) {
+            cpulist = g_new0(char *, virBitmapCountBits(vcpus) + 1);
+            for (i = 0; i < nvcpus; i++) {
+                if (VIR_CPU_USED(vcpumap, i) == 0)
+                    continue;
+
+                cpulist[offset++] = g_strdup_printf("%zu", i);
+            }
+        }
+    }
+
+    ret = virshCommaStringListComplete(vcpuid, (const char **)cpulist);
+
+ cleanup:
+    virTypedParamsFree(params, nparams);
+    virshDomainFree(dom);
+    return ret;
+}
+
+
+char **
+virshDomainConsoleCompleter(vshControl *ctl,
+                            const vshCmd *cmd,
+                            unsigned int flags)
+{
+    virshControlPtr priv = ctl->privData;
+    g_autoptr(xmlDoc) xmldoc = NULL;
+    g_autoptr(xmlXPathContext) ctxt = NULL;
+    int nserials;
+    int nparallels;
+    g_autofree xmlNodePtr *serials = NULL;
+    g_autofree xmlNodePtr *parallels = NULL;
+    size_t i;
+    size_t offset = 0;
+    VIR_AUTOSTRINGLIST tmp = NULL;
+
+    virCheckFlags(0, NULL);
+
+    if (!priv->conn || virConnectIsAlive(priv->conn) <= 0)
+        return NULL;
+
+    if (virshDomainGetXML(ctl, cmd, 0, &xmldoc, &ctxt) < 0)
+        return NULL;
+
+    nserials = virXPathNodeSet("./devices/serial", ctxt, &serials);
+    if (nserials < 0)
+        return NULL;
+
+    nparallels = virXPathNodeSet("./devices/parallel", ctxt, &parallels);
+    if (nparallels < 0)
+        return NULL;
+
+    tmp = g_new0(char *, nserials + nparallels + 1);
+
+    for (i = 0; i < nserials + nparallels; i++) {
+        g_autofree char *type = NULL;
+
+
+        if (i < nserials)
+            ctxt->node = serials[i];
+        else
+            ctxt->node = parallels[i - nserials];
+
+        type = virXPathString("string(./@type)", ctxt);
+        if (STRNEQ(type, "pty"))
+            continue;
+
+        tmp[offset++] = virXPathString("string(./alias/@name)", ctxt);
+    }
+
+    return g_steal_pointer(&tmp);
+}
+
+
+char **
+virshDomainSignalCompleter(vshControl *ctl G_GNUC_UNUSED,
+                           const vshCmd *cmd G_GNUC_UNUSED,
+                           unsigned int flags)
+{
+    VIR_AUTOSTRINGLIST tmp = NULL;
+    size_t i = 0;
+
+    virCheckFlags(0, NULL);
+
+    tmp = g_new0(char *, VIR_DOMAIN_PROCESS_SIGNAL_LAST + 1);
+
+    for (i = 0; i < VIR_DOMAIN_PROCESS_SIGNAL_LAST; i++) {
+        const char *name = virDomainProcessSignalTypeToString(i);
+        tmp[i] = g_strdup(name);
+    }
+
+    return g_steal_pointer(&tmp);
+}
+
+
+char **
+virshDomainLifecycleCompleter(vshControl *ctl G_GNUC_UNUSED,
+                              const vshCmd *cmd G_GNUC_UNUSED,
+                              unsigned int flags)
+{
+    VIR_AUTOSTRINGLIST tmp = NULL;
+    size_t i = 0;
+
+    virCheckFlags(0, NULL);
+
+    tmp = g_new0(char *, VIR_DOMAIN_LIFECYCLE_LAST + 1);
+
+    for (i = 0; i < VIR_DOMAIN_LIFECYCLE_LAST; i++) {
+        const char *name = virDomainLifecycleTypeToString(i);
+        tmp[i] = g_strdup(name);
+    }
+
+    return g_steal_pointer(&tmp);
+}
+
+
+char **
+virshDomainLifecycleActionCompleter(vshControl *ctl G_GNUC_UNUSED,
+                                    const vshCmd *cmd G_GNUC_UNUSED,
+                                    unsigned int flags)
+{
+    VIR_AUTOSTRINGLIST tmp = NULL;
+    size_t i = 0;
+
+    virCheckFlags(0, NULL);
+
+    tmp = g_new0(char *, VIR_DOMAIN_LIFECYCLE_ACTION_LAST + 1);
+
+    for (i = 0; i < VIR_DOMAIN_LIFECYCLE_ACTION_LAST; i++) {
+        const char *action = virDomainLifecycleActionTypeToString(i);
+        tmp[i] = g_strdup(action);
+    }
+
+    return g_steal_pointer(&tmp);
+}
